{
  let element = document.getElementById("fld_show_phpinfo");
  if (element != null ) {
    element.addEventListener('click', function() {
      toggle_visible('fld_show_phpinfo','=phpinfo');
    });
  }
}
